{***(V.K.)**********************************************************************
	IEHelper - sceleton BHO (IE's event sinking and manipulation)
  implementation in OP.

  Developed by Vadim Krochak (based on IEHelper VC++ MSDN sample).
*******************************************************************************}

unit IEHelperOP_IMPL;

interface

uses
  Windows, ActiveX, ComObj,
  {$IFDEF USE_FORM}
  Events_Form_U,
  {$ENDIF}
  SHDocVw_TLB;

type

	//(VK): Object factory:
  TIEHelperOPFactory = class(TComObjectFactory)
  private
    procedure AddKeys;
    procedure RemoveKeys;
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;


  TIEHelperOP = class(TComObject, IObjectWithSite, IDispatch)
  private
  	FConnection: Longint;
  	FWebBrowser: IWebBrowser2;
    FExplorerIsHost: Boolean;

    {$IFDEF USE_FORM}
    FEventsDlg: TEvents_Form;
    {$ENDIF}

    procedure Connect2Browser(const Connect: Boolean = True);
		procedure InternalLogEvent(const Msg: String);
    procedure SpyBeforeNavigate(Params: Pointer);

  protected
    {IObjectWithSite methods}
    function SetSite(const pUnkSite: IUnknown ):HResult; stdcall;
    function GetSite(const riid: TIID; out site: IUnknown):HResult; stdcall;
    {IDispatch methods}
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;

  public
  	procedure Initialize(); override;
    destructor Destroy(); override;

  end;

const
  Class_IEHelperOP: TGUID = '{49E0E0F0-5C30-11D4-945D-000000000000}';

//******************************************************************************
//******************************************************************************

implementation

uses ComServ, SysUtils, Registry, ExDispID, DOMAccess;

resourcestring

	BHO_ROOT_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Browser Helper Objects\';

  RESTRICTED_URL = 'http://www.sex.com';
  ALTERNATE_URL  = 'http://www.borland.com';


//******************************************************************************
//          TIEHelperOPFactory implementation
//******************************************************************************

procedure TIEHelperOPFactory.AddKeys;
begin

  with TRegistry.Create() do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    if OpenKey( BHO_ROOT_KEY + GUIDToString(Class_IEHelperOP), True)
      then CloseKey;
  finally
    Free();
  end;// try-finally
  
end;

//******************************************************************************
procedure TIEHelperOPFactory.RemoveKeys;
begin

  with TRegistry.Create() do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    DeleteKey(BHO_ROOT_KEY + GUIDToString(Class_IEHelperOP));
  finally
    Free();
  end;// try-finally

end;

//******************************************************************************
procedure TIEHelperOPFactory.UpdateRegistry(Register: Boolean);
begin
  inherited;

  if Register then
  	Self.AddKeys()
  else Self.RemoveKeys();

end;

//******************************************************************************
//          TIEHelperOP implementation
//******************************************************************************

procedure TIEHelperOP.Initialize;
begin

	inherited;
  {$IFDEF USE_FORM}
  Self.FEventsDlg := TEvents_Form.Create(nil);
  {$ENDIF}

end;

//******************************************************************************
destructor TIEHelperOP.Destroy;
begin

  {$IFDEF USE_FORM}
	Self.FEventsDlg.Free;
  {$ENDIF}
	inherited;
  
end;

{***(V.K.)**********************************************************************
  Procedure  :Connect2Browser
  Purpose    :connects/disconnects to/from IE to sink/unsink events
  Parameters :Connect
*******************************************************************************}
procedure TIEHelperOP.Connect2Browser(const Connect: Boolean);
var
  HostModuleName: array[0..255] of Char;
begin

	if Connect then
  begin
    InterfaceConnect(FWebBrowser, DWebBrowserEvents2, Self, FConnection);

    //(VK):check who is loading IEHelper:
    GetModuleFileName(0, HostModuleName, 256);
    MessageBox(0, @HostModuleName, 'IEHelperOP is connected to:', MB_OK);
   	Self.FExplorerIsHost := LowerCase(ExtractFileName(HostModuleName)) = 'explorer.exe';

    {$IFDEF USE_FORM}
		Self.FEventsDlg.WebBrowser := FWebBrowser;
    Self.FEventsDlg.Show();
    {$ENDIF}

  end
  else
	  InterfaceDisconnect(FWebBrowser, DWebBrowserEvents2, FConnection);

end;

{***(V.K.)**********************************************************************
  Procedure  :SpyBeforeNavigate
  Purpose    :spyies url before navigate, stops IE from navigating if url
  						restricted
  Parameters :Params - pointer to Invoke's Params
*******************************************************************************}
procedure TIEHelperOP.SpyBeforeNavigate(Params: Pointer);
begin

  //(VK):param. count checking:
  if PDispParams(Params).cArgs >= 5 then
  begin
    //(VK): show some output - url to navigate to is in rgvarg[5].pvarVal :
    Self.InternalLogEvent('BEFORENAVIGATE2: ' + PDispParams(Params)^.rgvarg[5].pvarVal^);

    //(VK): stop navigation:
    if PDispParams(Params)^.rgvarg[5].pvarVal^ = RESTRICTED_URL then
    begin
    	{**(V.K.)****************************************
        to stop IE from navigation this sequence must
        be performed:
        1 - canceling navigation
        2 - stopping IE
      ************************************************}
    	PDispParams(Params)^.rgvarg[0].pbVal^ := 1; // Cancel flag = True
      try
        Self.FWebBrowser.Stop();
        Self.FWebBrowser.Navigate(ALTERNATE_URL, EmptyParam, EmptyParam, EmptyParam, EmptyParam);
      except
        on E: Exception do
        begin
        	Windows.MessageBox(0, PChar(Format('Exception raised: %s', [E.Message])), 'IEHelperOP', MB_OK);
        end;
      end;// try-except
    end;// if
  end;// if

end;

{***(V.K.)**********************************************************************
  Function   :SetSite
  Purpose    :IObjectWithSite's SetSite method implementation.
  Parameters :See M$ documentation
  Returns    :
*******************************************************************************}
function TIEHelperOP.SetSite(const pUnkSite: IUnknown): HResult;
begin

	if Assigned(pUnkSite) then
  begin
    Self.FWebBrowser := pUnkSite as IWebBrowser2;
    Self.Connect2Browser();
		Result := S_OK;
  end
  else
		Result := E_FAIL;

end;

{***(V.K.)**********************************************************************
  Function   :GetSite
  Purpose    :IObjectWithSite's GetSite method implementation.
  Parameters :See M$ documentation
  Returns    :
*******************************************************************************}
function TIEHelperOP.GetSite(const riid: TIID; out site: IUnknown): HResult;
begin

  site := nil;
	Result := E_FAIL;
  
end;

{***(V.K.)**********************************************************************
	Sceleton implementation of IDispatch methods:
*******************************************************************************}
function TIEHelperOP.GetTypeInfo(Index, LocaleID: Integer;
	out TypeInfo): HResult;
begin

  Result := E_NOTIMPL;
  
end;

//******************************************************************************
function TIEHelperOP.GetTypeInfoCount(out Count: Integer): HResult;
begin

  Count  := 0;
	Result := S_OK;
  
end;

//******************************************************************************
function TIEHelperOP.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin

  Result := E_NOTIMPL;
  
end;

{***(V.K.)**********************************************************************
  Function   :Invoke
  Purpose    :IDispatch's Invoke implementation
  Parameters :See MS Help
  Returns    :
*******************************************************************************}
function TIEHelperOP.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin

	case DispID of
    DISPID_ONQUIT, // IE exit fired by DISPID_ONQUIT Event ID
    DISPID_QUIT: Self.Connect2Browser(False);

    {**(V.K.)****************************************
      DISPID_BEFORENAVIGATE2 - relates to
      BeforeNavigate2 event that defined as follows
      (See MSDN documentation 4 DWebBrowserEvents2 Interface):

      void BeforeNavigate2(
        IDispatch *pDisp,    			--> rgvarg[6]
        VARIANT *URL,        			--> rgvarg[5]
        VARIANT *Flags,      			--> rgvarg[4]
        VARIANT *TargetFrameName, --> rgvarg[3]
        VARIANT *PostData,   			--> rgvarg[2]
        VARIANT *Headers,    			--> rgvarg[1]
        VARIANT_BOOL *Cancel 			--> rgvarg[0]
      );

      Each parameter appears in rgvarg[] in reverse
      order(see mappings above).
    ************************************************}
    DISPID_BEFORENAVIGATE2:
      Self.SpyBeforeNavigate(Pointer(@Params));

    DISPID_DOCUMENTCOMPLETE: begin
      InternalLogEvent('DOCUMENTCOMPLETE -> url: ' + TDispParams(Params).rgvarg[0].pvarVal^ );
      //(VK):show navigation history count:
      if not Self.FExplorerIsHost then
      	InternalLogEvent('History count is: ' + IntToStr(HistoryCount(FWebBrowser)));
    end;

    {**(V.K.)****************************************
      Other events, 4 complete list of DispIDs see
      ExDispID unit.
    ************************************************}
    DISPID_STATUSTEXTCHANGE			: InternalLogEvent('STATUSTEXTCHANGE');
    DISPID_DOWNLOADCOMPLETE			: InternalLogEvent('DOWNLOADCOMPLETE');
    DISPID_COMMANDSTATECHANGE		: InternalLogEvent('COMMANDSTATECHANGE');
    DISPID_DOWNLOADBEGIN      	: InternalLogEvent('DOWNLOADBEGIN');
    DISPID_PROGRESSCHANGE    		: InternalLogEvent('PROGRESSCHANGE');
    DISPID_WINDOWMOVE         	: InternalLogEvent('WINDOWMOVE');
    DISPID_WINDOWRESIZE       	: InternalLogEvent('WINDOWRESIZE');
    DISPID_WINDOWACTIVATE     	: InternalLogEvent('WINDOWACTIVATE');
    DISPID_PROPERTYCHANGE     	: InternalLogEvent('PROPERTYCHANGE');
    DISPID_TITLECHANGE        	: InternalLogEvent('TITLECHANGE');
    DISPID_NAVIGATECOMPLETE2    : InternalLogEvent('NAVIGATECOMPLETE2');
    DISPID_ONVISIBLE            : InternalLogEvent('ONVISIBLE');
    DISPID_ONTOOLBAR            : InternalLogEvent('ONTOOLBAR');
    DISPID_ONMENUBAR            : InternalLogEvent('ONMENUBAR');
    DISPID_ONSTATUSBAR          : InternalLogEvent('ONSTATUSBAR');
    DISPID_ONFULLSCREEN         : InternalLogEvent('ONFULLSCREEN');
    DISPID_ONTHEATERMODE        : InternalLogEvent('ONTHEATERMODE');
    DISPID_ONADDRESSBAR         : InternalLogEvent('ONADDRESSBAR');

  else InternalLogEvent('Unknown DispID: ' + IntToStr(DispID));
  end;// case

	Result := S_OK;
  
end;

{***(V.K.)**********************************************************************
  Procedure  :InternalLogEvent
  Purpose    :logs event
  Parameters :Msg - log message
*******************************************************************************}
procedure TIEHelperOP.InternalLogEvent(const Msg: String);
begin

  {$IFDEF USE_FORM}
  FEventsDlg.LogEvent(Msg);
  {$ELSE}
  OutputDebugString(PChar(Msg));
  {$ENDIF}
  
end;

//******************************************************************************
//******************************************************************************

initialization
  TIEHelperOPFactory.Create(ComServer, TIEHelperOP, Class_IEHelperOP,
    'IEHelperOP', 'IE 4.x-5.x BHO in ObjectPascal', ciMultiInstance, tmApartment);

end.////////////////////////////////////////////////////////////////////////////
